$ifndef _HPWM_16F150X_INC_
$define _HPWM_16F150X_INC_
'
' Replacement routine for the compiler's HPWM library
' For devices 16F1503, 16F1507, 16F1508, and 16F1509
'
' Written for the Proton BASIC compiler, by Les Johnson
'
$if _device <> _16F1503 And _device <> _16F1507 And _device <> _16F1508 And _device <> _16F1509
    $error "Library HPWM_16F150X.inc is only suitable for devices 16F1503, 16F1507, 16F1508, and 16F1509"
$endif

    #Disable HPWM                      ' Disable the compiler's HPWM library routine
    #Enable Multiply, Divide           ' Enable the compiler's 16-bit Divide and Multiply library routines
'
' Create some compile system variables
'
    Dim PP0  As Byte System
    Dim PP0H As Byte System
    Dim PP1  As Byte System
    Dim PP1H As Byte System
    Dim PP2  As Byte System
    Dim PP2H As Byte System
    Dim PP3  As Byte System
    Dim PP3H As Byte System
    Dim PP4H As Byte System
    Dim PP6H As Byte System
    Dim GEN  As Byte System
'
' Make some aliases to the compiler variables
'
    Dim HPWM_bChannel          As PP4H                      ' Holds the channel
    Dim HPWM_bXtalFrequency    As PP0                       ' \
    Dim HPWM_bXtalFrequencyH   As PP0H                      ' | Used to load the fOSC into
    Dim HPWM_bXtalFrequencyHH  As PP2                       ' |
    Dim HPWM_bXtalFrequencyHHH As PP2H                      ' |
    Dim HPWM_dXtalFrequency    As HPWM_bXtalFrequency.Dword ' /

    Dim HPWM_wFSR0 As FSR0L.Word                            ' Create a 16-bit SFR from 2 8-bit SFRs
'--------------------------------------------------------------------------------

    GoTo HPWM_16F150X_Main                                  ' Jump over the routine

'--------------------------------------------------------------------------------
' Divide PP0\H by 4 by rotation
' Input     : PP0\H
' Output    : PP0\H
' Notes     : None
'
$define HPWM_DivideBy4() GoSub _HPWM_DivideBy4_

_HPWM_DivideBy4_:
    Ror PP0H
    Ror PP0
    Ror PP0H
    Ror PP0
    Return

'--------------------------------------------------------------------------------
' Output a Pulse Width Modulated signal from the PWM peripherals
' Input     : WREG = Channel 1 - 4
'           : PP1\H = Frequency
'           : GEN = Duty cycle (0 to 255)
' Output    : None
' Notes     : Replaces the compiler's library routine for devices:
'           : PIC16F1503, PIC16F1507, PIC16F1508, and PIC16F1509
'
__HPWM_:
    HPWM_bChannel = WREG                            ' Place the channel into a variable
    If HPWM_bChannel = 1 Then                       ' Are we accessing channel 1?
        HPWM_wFSR0 = AddressOf(PWM1DCL)             ' Yes. So load FSR0L\H with the address of PWM1DCL
        Output PORTC.5                              ' Make HPWM 1 pin an output
    ElseIf HPWM_bChannel = 2 Then                   ' Are we accessing channel 2?
        HPWM_wFSR0 = AddressOf(PWM2DCL)             ' Yes. So load FSR0L\H with the address of PWM2DCL
        Output PORTC.3                              ' Make HPWM 2 pin an output
    ElseIf HPWM_bChannel = 3 Then                   ' Are we accessing channel 3?
        HPWM_wFSR0 = AddressOf(PWM3DCL)             ' Yes. So load FSR0L\H with the address of PWM3DCL
        Output PORTA.2                              ' Make HPWM 3 pin an output
    ElseIf HPWM_bChannel = 4 Then                   ' Are we accessing channel 4?
        HPWM_wFSR0 = AddressOf(PWM4DCL)             ' Yes. So load FSR0L\H with the address of PWM4DCL
        Output PORTC.1                              ' Make HPWM 4 pin an output
    Else                                            ' Otherwise... Not a valid channel. So...
        Return                                      ' Exit the routine
    EndIf

HPWM_CalcFrequency_:
    HPWM_dXtalFrequency = ((_xtal * 1000000) / 4)
    Movlp (__Divide_Int_U1616_ >> 8)
    Call (__Divide_Int_U1616_)                      ' Xtal / frequency
    T2CON = T2CON & 252                             ' Clear T2CON PS bits
    Movf PP0H,w
    If STATUSbits_Z = 1 Then Bra HPWM_Ready_
    Set T2CONbits_T2CKPS0
    HPWM_DivideBy4()

    Andlw 252
    If STATUSbits_Z = 1 Then Bra HPWM_Ready_
    Clear T2CONbits_T2CKPS0
    Set T2CONbits_T2CKPS1
    HPWM_DivideBy4()

    Andlw 240
    If STATUSbits_Z = 1 Then Bra HPWM_Ready_
    Set T2CONbits_T2CKPS0
    HPWM_DivideBy4()

HPWM_Ready_:
    Set T2CONbits_TMR2ON                            ' Start Timer2
    Decf PP0,w
    Movwf PR2
    Incfsz GEN,w
    Bra HPWM_Multiply_
    Movlw 255
    Movwf PP2
    Movwf PP2H
    Bra HPWM_LoadSFRs_

HPWM_Multiply_:
    Movf GEN,w
    Movwf PP2H
    Clear PP2
    Movwf PP3
    Clear PP3H
    Incf PR2,w
    If STATUSbits_Z = 0 Then
        Movwf PP1
        Clear PP1H
        Movlp (__Multiply_U1616_ >> 8)
        Call (__Multiply_U1616_)
    EndIf
'
' Load the PWM SFRs with the values required
'
HPWM_LoadSFRs_:
    WREG = PP2                                      ' \
    Movwi INDF0++                                   ' / Place a value into PWMxDCL
    WREG = PP2H                                     ' \
    Movwi INDF0++                                   ' / Place a value into PWMxDCH
    INDF0 = INDF0 | 192                             ' Place a value into PWMxCON
    Return

'--------------------------------------------------------------
HPWM_16F150X_Main:

$endif      ' _HPWM_16F150X_INC_


